x86_32 Darwin ABI: Treat empty unions like empty structures.
 - Current return-arguments-32 status: 15/1000 failures


git-svn-id: https://llvm.org/svn/llvm-project/cfe/trunk@68132 91177308-0d34-0410-b5e6-96231b3b80d8
diff --git a/test/CodeGen/x86_32-arguments.c b/test/CodeGen/x86_32-arguments.c
index 632949e..9334895 100644
--- a/test/CodeGen/x86_32-arguments.c
+++ b/test/CodeGen/x86_32-arguments.c
@@ -15,8 +15,6 @@
 // tests. This should be the same as @f8_2.
 // RUN: grep 'define void @f9_2(%.truct.s9\* byval %a0)' %t &&
 
-// RUN: true
-
 char f0(void) {
 }
 
@@ -65,3 +63,13 @@
 void f9_2(struct s9 a0) {
 }
 
+// Return of small structures and unions...
+
+// RUN: grep 'float @f10()' %t &&
+struct s10 {
+  union { };
+  float f;
+} f10(void) {}
+
+// RUN: true
+
diff --git a/lib/CodeGen/CGCall.cpp b/lib/CodeGen/CGCall.cpp
index 8217b04..291e476 100644
--- a/lib/CodeGen/CGCall.cpp
+++ b/lib/CodeGen/CGCall.cpp
@@ -140,11 +140,11 @@
 
 /***/
 
 /// isEmptyStruct - Return true iff a structure has no non-empty
 /// members. Note that a structure with a flexible array member is not
 /// considered empty.
-static bool isEmptyStruct(QualType T) {
-  const RecordType *RT = T->getAsStructureType();
+static bool isEmptyRecord(QualType T) {
+  const RecordType *RT = T->getAsRecordType();
   if (!RT)
     return 0;
   const RecordDecl *RD = RT->getDecl();
@@ -153,7 +153,7 @@
   for (RecordDecl::field_iterator i = RD->field_begin(), 
          e = RD->field_end(); i != e; ++i) {
     const FieldDecl *FD = *i;
-    if (!isEmptyStruct(FD->getType()))
+    if (!isEmptyRecord(FD->getType()))
       return false;
   }
   return true;
@@ -182,7 +182,7 @@
     const FieldDecl *FD = *i;
     QualType FT = FD->getType();
 
-    if (isEmptyStruct(FT)) {
+    if (isEmptyRecord(FT)) {
       // Ignore
     } else if (Found) {
       return 0;
@@ -140,11 +140,11 @@
 /***/
 
-/// isEmptyStruct - Return true iff a structure has no non-empty
+/// isEmptyRecord - Return true iff a structure has no non-empty
 /// members. Note that a structure with a flexible array member is not
 /// considered empty.